@@ -13,9 +13,6 @@
 #define NETLINK_BASIC_H_
 
 #include <netlink/netlink.h>
-#include <netlink/route/classifier.h>
-#include <netlink/route/cls/ematch.h>
-#include <netlink/route/action.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -26,8 +23,6 @@ extern uint32_t			rtnl_basic_get_target(struct rtnl_cls *);
 extern void			rtnl_basic_set_ematch(struct rtnl_cls *,
 						      struct rtnl_ematch_tree *);
 extern struct rtnl_ematch_tree *rtnl_basic_get_ematch(struct rtnl_cls *);
-extern int rtnl_basic_add_action(struct rtnl_cls *, struct rtnl_act *);
-extern int rtnl_basic_del_action(struct rtnl_cls *, struct rtnl_act *);
 
 #ifdef __cplusplus
 }
