@@ -16,13 +16,12 @@
  * @{
  */
 
-#include <netlink-private/netlink.h>
+#include <netlink-local.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/route/rtnl.h>
 #include <netlink/route/rule.h>
 #include <inttypes.h>
-#include <linux/fib_rules.h>
 
 /** @cond SKIP */
 #define RULE_ATTR_FAMILY	0x0001
@@ -246,12 +245,12 @@ static void rule_dump_stats(struct nl_object *obj, struct nl_dump_params *p)
 
 #define RULE_ATTR_FLAGS		0x0008
 
-static uint64_t rule_compare(struct nl_object *_a, struct nl_object *_b,
-			     uint64_t attrs, int flags)
+static int rule_compare(struct nl_object *_a, struct nl_object *_b,
+			uint32_t attrs, int flags)
 {
 	struct rtnl_rule *a = (struct rtnl_rule *) _a;
 	struct rtnl_rule *b = (struct rtnl_rule *) _b;
-	uint64_t diff = 0;
+	int diff = 0;
 
 #define RULE_DIFF(ATTR, EXPR) ATTR_DIFF(attrs, RULE_ATTR_##ATTR, a, b, EXPR)
 
@@ -275,19 +274,19 @@ static uint64_t rule_compare(struct nl_object *_a, struct nl_object *_b,
 }
 
 static const struct trans_tbl rule_attrs[] = {
-	__ADD(RULE_ATTR_FAMILY, family),
-	__ADD(RULE_ATTR_TABLE, table),
-	__ADD(RULE_ATTR_ACTION, action),
-	__ADD(RULE_ATTR_IIFNAME, iifname),
-	__ADD(RULE_ATTR_OIFNAME, oifname),
-	__ADD(RULE_ATTR_PRIO, prio),
-	__ADD(RULE_ATTR_MARK, mark),
-	__ADD(RULE_ATTR_MASK, mask),
-	__ADD(RULE_ATTR_GOTO, goto),
-	__ADD(RULE_ATTR_SRC, src),
-	__ADD(RULE_ATTR_DST, dst),
-	__ADD(RULE_ATTR_DSFIELD, dsfield),
-	__ADD(RULE_ATTR_FLOW, flow),
+	__ADD(RULE_ATTR_FAMILY, family)
+	__ADD(RULE_ATTR_TABLE, table)
+	__ADD(RULE_ATTR_ACTION, action)
+	__ADD(RULE_ATTR_IIFNAME, iifname)
+	__ADD(RULE_ATTR_OIFNAME, oifname)
+	__ADD(RULE_ATTR_PRIO, prio)
+	__ADD(RULE_ATTR_MARK, mark)
+	__ADD(RULE_ATTR_MASK, mask)
+	__ADD(RULE_ATTR_GOTO, goto)
+	__ADD(RULE_ATTR_SRC, src)
+	__ADD(RULE_ATTR_DST, dst)
+	__ADD(RULE_ATTR_DSFIELD, dsfield)
+	__ADD(RULE_ATTR_FLOW, flow)
 };
 
 static char *rule_attrs2str(int attrs, char *buf, size_t len)
@@ -384,10 +383,6 @@ static int build_rule_msg(struct rtnl_rule *tmpl, int cmd, int flags,
 	if (nlmsg_append(msg, &frh, sizeof(frh), NLMSG_ALIGNTO) < 0)
 		goto nla_put_failure;
 
-	/* Additional table attribute replacing the 8bit in the header, was
-	 * required to allow more than 256 tables. */
-	NLA_PUT_U32(msg, FRA_TABLE, tmpl->r_table);
-
 	if (tmpl->ce_mask & RULE_ATTR_SRC)
 		NLA_PUT_ADDR(msg, FRA_SRC, tmpl->r_src);
 
